#!/usr/bin/perl

use strict;
use CGI();
use Data::Dumper;

my $wcald;
my $wconf;
my $uconf;
my $gconf;

my $topic = "";

$::CWCAL_VERSION = "1.2b5";

$::DEFAULTS = {
    'access file name' => '.htaccess',
    'address' => '127.0.0.1',
    'base url' => '/wcal',
    'clock' => '24-hour',
    'data id' => '',
    'database directory' => '/var/wcal',
    'date format' => 2,
    'day precision' => 1,
    'default lock' => 'false',
    'default priv' => 'false',
    'default todo' => 'true',
    'detailed week' => 'no',
    'demo' => 'no',
    'disable fusion' => 'false',
    'disable purge' => 'false',
    'event length' => 4,
    'event precision' => 15,
    'first day' => 'monday',
    'first hour' => 8,
    'footer' => '',
    'frame height' => '',
    'frame width' => '',
    'last hour' => 18,
    'long name' => '',
    'mail command' => 'mail -s',
    'organization' => '',
    'overlap' => 'true',
    'password' => '',
    'port' => 13134,
    'read only' => 'false',
    'refresh delay' => 0,
    'remote cal' => 'all',
    'remote lang' => 'en',
    'remote mail' => 'root',
    'show day time' => 'no',
    'show weekend' => 'yes',
    'superuser' => '',
    'theme' => '',
    'top left corner' => '',
    'user order' => '',
    'webserver port' => '',
    'week copy' => 'no',
};

# internationalization
sub __
{
    my $str = shift;
    return $::msgs->{$str}->{$::REMOTE_LANG} || "$str*";
}

# html header
sub cw_header
{
    my $title = shift;
    my $www = shift;
    my $theme = shift;

    my %params = (-title=>"$title");
   
    print CGI::header();
    if ($www)
    {
	$params{-background} = "../background.jpg";
	$params{-bgcolor} = "#edd4fb";
    }
    else
    {
	$params{-background} = "images/background.jpg";
	$params{-bgcolor} = "#d1e3fa";
    }
    if ($theme)
    {
	if ($theme->{textcolor})
	{
	    $params{-text} = $theme->{textcolor};
	}
	if ($theme->{linkcolor})
	{
	    $params{-link} = $theme->{linkcolor};
	}
	if ($theme->{vlinkcolor})
	{
	    $params{-vlink} = $theme->{vlinkcolor};
	}
	if ($theme->{alinkcolor})
	{
	    $params{-alink} = $theme->{alinkcolor};
	}
    }
    print CGI::start_html(%params);
}

# html footer
sub cw_footer
{
    print CGI->end_html();
}

sub cw_read_wcald
{
    open(WCALD, $::WCAL_BIN) or die "Can't open $::WCAL_BIN for reading: $!";
    my $line;
    my $key;
    my $val;

    my $wcald = {};
    while ($line = <WCALD>)
    {
	if ($line =~ /^\s*sub/)
	{
	    last;
	}
	elsif ($line =~ /^\s*\$\:\:(\S+)\s*=\s*(.*\S);/)
	{
	    $key = $1;
	    $val = $2;
	    $val =~ s/^(\'|\")//;
	    $val =~ s/(\'|\")$//;
	    $wcald->{$key} = $val;
	}
    }
    close(WCALD);
    return $wcald;
}

sub cw_get_wcald
{
    if (!$wcald)
    {
	$wcald = cw_read_wcald();
    }
    return $wcald;
}

sub cw_read_wconf
{
    my $wcald = cw_get_wcald();
    open(CONF, $wcald->{CONF_FILE}) or die "Can't open $wcald->{CONF_FILE} for reading: $!";
    my $line;
    my $key;
    my $val;

    my $wconf = {};
    my @users = ();
    my @groups = ();

    while ($line = <CONF>)
    {
	if ($line =~ /^\s*\#/)
	{
	    next;
	}
	elsif ($line =~ /^\s*access\s+file/)
	{
	    $val = "";
	    while ($line = <CONF>)
	    {
		if ($line =~ /^\s*end\s+access\s+file/)
		{
		    last;
		}
		$line =~ s/^\s*//;
		$val .= $line;
	    }
	    $wconf->{"access file"} = $val;
	}
	elsif ($line =~ /^\s*([^=]*\S)\s*=\s*(.*\S)/)
	{
	    $key = $1;
	    $val = $2;
	    if ($key eq "user" || $key eq "group")
	    {
		last;
	    }
	    $wconf->{$key} = $val;
	}
    }
    my $cuser;
    my $cgroup;
    $uconf = {};
    $gconf = {};
    my $conf_ref = $wconf;

    while ($line)
    {
	if ($line =~ /^\s*user\s*=\s*(.*\S)/)
	{
	    $cuser = $1;
	    if (!$conf_ref->{users})
	    {
		$conf_ref->{users} = [];
	    }
	    push @{$conf_ref->{users}}, $cuser;
	    $conf_ref = {};
	    if ($cgroup)
	    {
		$conf_ref->{group} = $cgroup;
	    }
	}
	elsif ($line =~ /^\s*end\s+user/)
	{
	    $uconf->{$cuser} = $conf_ref;
	    $cuser= "";
	    if ($cgroup)
	    {
		$conf_ref = $gconf->{$cgroup};
	    }
	    else
	    {
		$conf_ref = $wconf;
	    }
	}
	elsif ($line =~ /^\s*group\s*=\s*(.*\S)/)
	{
	    my $fgroup;
	    if ($cgroup)
	    {
		push @groups, $cgroup;
		$fgroup = $cgroup;
	    }

	    $cgroup = $1;
	    if (!$conf_ref->{groups})
	    {
		$conf_ref->{groups} = [];
	    }
	    push @{$conf_ref->{groups}}, $cgroup;
	    $conf_ref = {};
	    if ($cgroup)
	    {
		$conf_ref->{group} = $fgroup;
	    }
	    $gconf->{$cgroup} = $conf_ref;
	}
	elsif ($line =~ /^\s*end\s+group/)
	{
	    $cgroup = pop @groups;
	    if ($cgroup)
	    {
		$conf_ref = $gconf->{$cgroup};
	    }
	    else
	    {
		$conf_ref = $wconf;
	    }
	}
	elsif (($cuser || $cgroup) && $line =~ /^\s*access\s+file/)
	{
	    $val = "";
	    while ($line = <CONF>)
	    {
		if ($line =~ /^\s*end\s+access\s+file/)
		{
		    last;
		}
		$line =~ s/^\s*//;
		$val .= $line;
	    }
	    $conf_ref->{"access file"} = $val;
	}
	elsif (($cuser || $cgroup) && $line =~ /^\s*([^=]*\S)\s*=\s*(.*\S)/)
	{
	    $key = $1;
	    $val = $2;
	    $conf_ref->{$key} = $val;
	}

	$line = <CONF>;
    }
    close(CONF);

    return $wconf;
}

sub cw_get_wconf
{
    if (!$wconf)
    {
	$wconf = cw_read_wconf();
    }
    return $wconf;
}

sub cw_get_uconf
{
    if (!$uconf)
    {
	$wconf = cw_read_wconf();
    }
    return $uconf;
}

sub cw_get_gconf
{
    if (!$gconf)
    {
	$wconf = cw_read_wconf();
    }
    return $gconf;
}

sub cw_write_conf
{
    my $wconf = shift;

    my $wcald = cw_get_wcald();
    open(CONF, ">/tmp/wcal.conf") or die "Can't open /tmp/wcal.conf for writing: $!";
    my $line;
    my $key;

    print CONF '# '.__("Gnr par CWcal")."\n";
    foreach $key (sort (keys(%{$wconf})))
    {
	if ($key eq "users" || $key eq "groups" || $key eq "group")
	{
	    next;
	}
	elsif ($key eq "access file")
	{
	    print CONF "\naccess file\n";
	    foreach $line (split(/\n/, $wconf->{$key}))
	    {
		print CONF "\t$line\n";
	    }
	    print CONF "end access file\n";
	}
	else
	{
	    print CONF "\n$key = $wconf->{$key}\n";
	}
    }
    open(CONF2, $wcald->{CONF_FILE}) or die "Can't open $wcald->{CONF_FILE} for reading: $!";
    while ($line = <CONF2>)
    {
	if ($line =~ /^\s*([^=]*\S)\s*=\s*(.*\S)/)
	{
	    $key = $1;
	    if ($key eq "user" || $key eq "group")
	    {
		print CONF "\n$line";
		last;
	    }
	}
    }
    while ($line = <CONF2>)
    {
	print CONF $line;
    }
    close(CONF2);
    close(CONF);

    system("\\cp /tmp/wcal.conf $wcald->{CONF_FILE}");
    chmod(0640, $wcald->{CONF_FILE});
}

sub cw_get_users
{
    my $conf_ref = shift;
    if ($conf_ref->{"users order"})
    {
	my %uids;
	my $user;
	my @users;
	if ($conf_ref->{users})
	{
	    foreach $user (@{$conf_ref->{users}})
	    {
		$uids{$user} = 1;
	    }
	}
	foreach $user (split(/\s+/,$conf_ref->{"users order"}))
	{
	    if ($uids{$user})
	    {
		push @users, $user;
		$uids{$user} = 0;
	    }
	}
	if ($conf_ref->{users})
	{
	    foreach $user (@{$conf_ref->{users}})
	    {
		if ($uids{$user})
		{
		    push @users, $user;
		}
	    }
	}
	return @users;
    }
    else
    {
	if ($conf_ref->{users})
	{
	    return @{$conf_ref->{users}};
	}
	else
	{
	    return ();
	}
    }
}

sub cw_get_groups
{
    my $conf_ref = shift;
    if ($conf_ref->{"groups order"})
    {
	my %gids;
	my $group;
	my @groups;
	if ($conf_ref->{groups})
	{
	    foreach $group (@{$conf_ref->{groups}})
	    {
		$gids{$group} = 1;
	    }
	}
	foreach $group (split(/\s+/,$conf_ref->{"groups order"}))
	{
	    if ($gids{$group})
	    {
		push @groups, $group;
		$gids{$group} = 0;
	    }
	}
	if ($conf_ref->{groups})
	{
	    foreach $group (@{$conf_ref->{groups}})
	    {
		if ($gids{$group})
		{
		    push @groups, $group;
		}
	    }
	}
	return @groups;
    }
    else
    {
	if ($conf_ref->{groups})
	{
	    return @{$conf_ref->{groups}};
	}
	else
	{
	    return ();
	}
    }
}

sub cw_write_uconf
{
    my $user = shift;
    my $uconf = shift;

    my $wcald = cw_get_wcald();
    open(CONF, ">/tmp/wcal.conf") or die "Can't open /tmp/wcal.conf for writing: $!";
    my $line;
    my $key;

    open(CONF2, $wcald->{CONF_FILE}) or die "Can't open $wcald->{CONF_FILE} for reading: $!";
    while ($line = <CONF2>)
    {
	if ($line =~ /^(\s*)user\s*=\s*$user\s*$/)
	{
	    my $spc = $1;
	    if ($uconf)
	    {
		print CONF $line;
		foreach $key (sort (keys(%{$uconf})))
		{
		    if ($key eq "users" || $key eq "groups" || $key eq "group")
		    {
			next;
		    }
		    elsif ($key eq "access file")
		    {
			print CONF $spc."\taccess file\n";
			foreach $line (split(/\n/, $uconf->{$key}))
			{
			    print CONF $spc."\t\t$line\n";
			}
			print CONF $spc."\tend access file\n";
		    }
		    else
		    {
			print CONF "$spc\t$key = $uconf->{$key}\n";
		    }
		}
	    }
	    while ($line = <CONF2>)
	    {
		if ($line =~ /^\s*end\s*user/)
		{
		    if ($uconf)
		    {
			print CONF $spc."end user\n";
		    }
		    last;
		}
	    }
	}
	else
	{
	    print CONF $line;
	}
    }
    close(CONF2);
    close(CONF);

    system("\\cp /tmp/wcal.conf $wcald->{CONF_FILE}");
    chmod(0640, $wcald->{CONF_FILE});
}

sub cw_write_gconf
{
    my $group = shift;
    my $gconf = shift;

    my $wcald = cw_get_wcald();
    open(CONF, ">/tmp/wcal.conf") or die "Can't open /tmp/wcal.conf for writing: $!";
    my $line;
    my $key;

    open(CONF2, $wcald->{CONF_FILE}) or die "Can't open $wcald->{CONF_FILE} for reading: $!";
    while ($line = <CONF2>)
    {
	if ($line =~ /^(\s*)group\s*=\s*$group\s*$/)
	{
	    my $spc = $1;
	    if ($gconf)
	    {
		print CONF $line;
		foreach $key (sort (keys(%{$gconf})))
		{
		    if ($key eq "users" || $key eq "groups" || $key eq "group")
		    {
			next;
		    }
		    elsif ($key eq "access file")
		    {
			print CONF $spc."\taccess file\n";
			foreach $line (split(/\n/, $gconf->{$key}))
			{
			    print CONF $spc."\t\t$line\n";
			}
			print CONF $spc."\tend access file\n";
		    }
		    else
		    {
			print CONF "$spc\t$key = $gconf->{$key}\n";
		    }
		}
	    }
	    my $pr = 0;
	    my $gr = 1;
	    while ($line = <CONF2>)
	    {
		if ($line =~ /^\s*user\s*=/)
		{
		    $pr = 1;
		}
		elsif ($line =~ /^\s*group\s*=/)
		{
		    $pr = 1;
		    $gr++;
		}
		if ($pr && $gconf)
		{
		    print CONF $line;
		}
		if ($line =~ /^\s*end\s+user/)
		{
		    if ($gr < 2)
		    {
			$pr = 0;
		    }
		}
		elsif ($line =~ /^\s*end\s+group/)
		{
		    $gr--;
		    if (!$gr)
		    {
			if ($gconf)
			{
			    print CONF $line;
			}
			last;
		    }
		    elsif ($gr < 2)
		    {
			$pr = 0;
		    }
		}
	    }
	}
	else
	{
	    print CONF $line;
	}
    }
    close(CONF2);
    close(CONF);

    system("\\cp /tmp/wcal.conf $wcald->{CONF_FILE}");
    chmod(0640, $wcald->{CONF_FILE});
}

sub cw_add_wcobj
{
    my $wcid = shift;
    my $objtype = shift;
    my $uconf = shift;
    my $group = shift;

    my $wcald = cw_get_wcald();
    open(CONF, ">/tmp/wcal.conf") or die "Can't open /tmp/wcal.conf for writing: $!";
    my $line;
    my $spc;
    my $key;

    open(CONF2, $wcald->{CONF_FILE}) or die "Can't open $wcald->{CONF_FILE} for reading: $!";
    while ($line = <CONF2>)
    {
	if ($group && $line =~ /^(\s*)group\s*=\s*$group\s*$/)
	{
	    $spc = "$1\t";
	    print CONF $line;
	    my $gr = 1;
	    while  ($line = <CONF2>)
	    {
		if ($line =~ /^\s*group\s*=/)
		{
		    $gr++;
		}
		elsif ($line =~ /^\s*end\s+group/)
		{
		    $gr--;
		    if (!$gr)
		    {
			print CONF $spc."$objtype=$wcid\n";
			foreach $key (sort (keys(%{$uconf})))
			{
			    if ($key eq "users" || $key eq "groups" || $key eq "group")
			    {
				next;
			    }
			    elsif ($key eq "access file")
			    {
				print CONF $spc."\taccess file\n";
				foreach $line (split(/\n/, $uconf->{$key}))
				{
				    print CONF $spc."\t\t$line\n";
				}
				print CONF $spc."\tend access file\n";
			    }
			    else
			    {
				print CONF "$spc\t$key = $uconf->{$key}\n";
			    }
			}
			print CONF $spc."end $objtype\n";
			print CONF $line;
			last;
		    }
		}
		print CONF $line;
	    }
	}
	else
	{
	    print CONF $line;
	}
    }
    if (!$group)
    {
	print CONF "$objtype=$wcid\n";
	foreach $key (sort (keys(%{$uconf})))
	{
	    if ($key eq "users" || $key eq "groups" || $key eq "group")
	    {
		next;
	    }
	    elsif ($key eq "access file")
	    {
		print CONF "\taccess file\n";
		foreach $line (split(/\n/, $uconf->{$key}))
		{
		    print CONF "\t\t$line\n";
		}
		print CONF "\tend access file\n";
	    }
	    else
	    {
		print CONF "$spc\t$key = $uconf->{$key}\n";
	    }
	}
	print CONF "end $objtype\n";
	
    }
    close(CONF2);
    close(CONF);

    system("\\cp /tmp/wcal.conf $wcald->{CONF_FILE}");
    chmod(0640, $wcald->{CONF_FILE});
}

sub cw_resource_link
{
    my $res = shift;
    my $grey = shift;
    my $bold = shift;
    my $helpurl = shift;

    my $col;
    if ($grey)
    {
	$col = '#666666';
    }
    else
    {
	$col = '#0000ff';
    }

    my $url = $helpurl || "help.cgi?res=".CGI::escape($res);
    if ($bold)
    {
	$res = "<b>$res</b>";
    }
    return '<a href="'.$url.'" onclick=\'win=window.open("'.$url.'", "help", "toolbar=no,menubar=no,scrollbars=yes,width=300,height=300,resizable=yes");win.focus();return false\'><font color="'.$col.'">'.$res."</font></a>";
}

sub cw_unquote
{
    my $str = shift;
    $str =~ s/\"/&quot;/g;
    $str =~ s/\'/&\#39;/g;
    $str =~ s/</&lt;/g;
    $str =~ s/>/&gt;/g;
    return $str;
}

sub cw_get_langs
{
    my %langs = ('en' => __('Anglais'),
		 'fr' => __('Franais'),
		 'de' => __('Allemand'),
		 'it' => __('Italien'),
		 'nl' => __('Nerlandais'),
		 'es' => __('Espagnol'),
		 'no' => __('Norvgien'),
		 'fi' => __('Finlandais'),
		 'pt' => __('Portugais')
		 );
    my $lang;
    my @values = ();
    my $value;
    foreach $lang (sort {$langs{$a} cmp $langs{$b}} keys(%langs))
    {
	$value = {};
	$value->{key} = $lang;
	$value->{str} =  $langs{$lang};
	push @values, $value;

    }
    return \@values;
}

sub cw_get_yes_no
{
    my @values = ();
    my $value;

    $value = {};
    $value->{key} = "yes";
    $value->{str} = __("Oui");
    push @values, $value;

    $value = {};
    $value->{key} = "no";
    $value->{str} = __("Non");
    push @values, $value;
    $value = {};

    return \@values;
}

sub cw_get_true_false
{
    my @values = ();
    my $value;

    $value = {};
    $value->{key} = "true";
    $value->{str} = __("Vrai");
    push @values, $value;

    $value = {};
    $value->{key} = "false";
    $value->{str} = __("Faux");
    push @values, $value;
    $value = {};

    return \@values;
}

sub cw_get_clock
{
    my @values = ();
    my $value;

    $value = {};
    $value->{key} = "24-hour";
    $value->{str} = __("24 heures");
    push @values, $value;

    $value = {};
    $value->{key} = "12-hour";
    $value->{str} = __("12 heures");
    push @values, $value;
    $value = {};

    return \@values;
}

sub cw_get_day_prec
{
    my @values = ();
    my $value;

    $value = {};
    $value->{key} = 1;
    $value->{str} = __("Heure");
    push @values, $value;

    $value = {};
    $value->{key} = 2;
    $value->{str} = __("Demi-heure");
    push @values, $value;
    $value = {};

    $value = {};
    $value->{key} = 4;
    $value->{str} = __("Quart d'heure");
    push @values, $value;

    return \@values;
}

sub cw_get_date_fmt
{
    my $day = __("Jour");
    my $month = __("Mois");
    my $year = __("Anne");

    my @values = ();
    my $value;

    $value = {};
    $value->{key} = 1;
    $value->{str} = "$day.$month.$year";
    push @values, $value;

    $value = {};
    $value->{key} = 2;
    $value->{str} = "$month/$day/$year";
    push @values, $value;

    $value = {};
    $value->{key} = 3;
    $value->{str} = "$day/$month/$year";
    push @values, $value;

    $value = {};
    $value->{key} = 4;
    $value->{str} = "$year/$month/$day";
    push @values, $value;

    $value = {};
    $value->{key} = 5;
    $value->{str} = "$year-$month-$day";
    push @values, $value;

    $value = {};
    $value->{key} = 6;
    $value->{str} = "$year$month$day";
    push @values, $value;

    return \@values;
}

sub cw_get_data_id
{
    my @values = ();
    my $value;

    my $uconf = cw_get_uconf();
    my $gconf = cw_get_gconf();
    my $user;
    my $uc;
    my $q = new CGI();
    my $cuser = $q->param("id");
    while (($user, $uc) = each(%{$uconf}))
    {
	if ($user eq $cuser)
	{
	    next;
	}
	if (!$uc->{"data id"} || $uc->{"data id"} eq $user)
	{
	    $value = {};
	    $value->{key} = $user;
	    $value->{str} = $uconf->{$user}->{"long name"};
	    my $conf_ref = $uconf->{$user};
	    my $group;
	    while ($group = $conf_ref->{group})
	    {
		$value->{str} = $gconf->{$group}->{"long name"}." / ".$value->{str};
		$conf_ref = $gconf->{$group};
	    }
	    push @values, $value;
	}
    }
    @values = sort {$a->{str} cmp $b->{str}} @values;
    return \@values;
}

sub cw_topic
{
    $topic = __(shift);
}

sub cw_field
{
    my $field = shift;
    my $conf = shift;
    my $q = shift;
    my $size = shift || 20;
    my $optfn = shift;
    my $nodef = shift;
    my $helpurl = shift;
    my $val;
    my $grey;
    my $values;
    my $value;
    my $def;

    if (defined $conf->{$field} || $q->param("all") || $nodef)
    {
	if (defined $conf->{$field})
	{
	    $val = $conf->{$field};
	    $grey = 0;
	}
	else
	{
	    $val = $::DEFAULTS->{$field};
	    $grey = 1;
	}

	if ($topic)
	{
	    print '<tr><td align="center"><font color="#aa44aa"><b>'.$topic.'</b></font></td><td><hr></td></tr>'."\n";
	}
	$topic = "";

	print "<tr>\n";
	print "<td align=right>".cw_resource_link($field, $grey, $nodef, $helpurl)." :</td>\n";
	print "<td>\n";
	if ($optfn)
	{
	    print '<select name="'.$field.'">'."\n";
	    $values = eval ("$optfn()");
	    foreach $value (@{$values})
	    {
		print '<option value="'.cw_unquote($value->{key}).'"';
		if ($value->{key} eq $val)
		{
		    print " selected";
		}
		if ($value->{key} eq $::DEFAULTS->{$field})
		{
		    $def = $value->{str};
		}
		print ">$value->{str}\n";
	    }
	    print "</select>\n";

	}
	else
	{
	    print '<input name="'.$field.'" size="'.$size.'" value="'.cw_unquote($val).'">'."\n";
	}
	if (!$grey && !$nodef)
	{
	    my $msg = sprintf(__("Slectionnez pour revenir  la valeur par dfaut (%s)"), $def ? $def : $::DEFAULTS->{$field});
	    print '<input type="checkbox" name="__'.$field.'" value="1"> <img src="images/reload.gif" alt="'.$msg.'" title="'.$msg.'">'."\n";
	}
	print "</td>\n";
	print "</tr>\n";
	return 1;
    }
    else
    {
	return 0;
    }
}

sub cw_tmpl_access
{
    my $wctype = shift;
    my $wcid = shift;

    my $authname;
    my $authacl;
    my $conf_ref;

    if (!$wctype)
    {
	$authname = "Wcal";
	$authacl = "valid-user";

    }
    else
    {
	if ($wctype eq "user")
	{
	    $conf_ref = cw_get_uconf();
	}
	elsif ($wctype eq "group")
	{
	    $conf_ref = cw_get_gconf();
	}
	else
	{
	    die "unknown type $wctype";
	}
	$authname = $conf_ref->{$wcid}->{"long name"};
	$authacl = "$wctype $wcid";
    }

    my $line;
    my $str;

    open(TMPL, "inc/cwaccess.tmpl") or die "Can't open cwaccess.tmpl for reading: $!";
    while ($line = <TMPL>)
    {
	$line =~ s/(AuthName\s*\")[^\"]*(\")/$1$authname$2/i;
	$line =~ s/(Require\s*).*\S/$1$authacl/i;
	$str .= $line;
    }
    return $str;
}

sub cw_get_theme
{
    my $id = shift;
    my $theme;
    my $wcald = cw_get_wcald();
    if (open(THM, "$wcald->{DIRECTORY}/themes/$id/theme.desc"))
    {
	my $old_slash = $/;
	undef $/;
	$theme = eval (<THM>);
	$/ = $old_slash;
	close(THM);
    }
    return $theme;
}

sub cw_write_theme
{
    my $theme = shift;
    my $wcald = cw_get_wcald();
    if (open(THM, ">$wcald->{DIRECTORY}/themes/$theme->{id}/theme.desc"))
    {
	$Data::Dumper::Terse = 1;
	print THM Dumper ($theme);
	close(THM);
    }
}

sub cw_themes
{
  my $wcald = cw_get_wcald();
  opendir(DIR, "$wcald->{DIRECTORY}/themes") || mkdir ("$wcald->{DIRECTORY}/themes", 0770);
  my $name;
  my $theme;
  my %themes = ();
  while ($name = readdir(DIR))
  {
      if ($name !~ /^\./)
      {
	  $theme = cw_get_theme($name);
	  if ($theme)
	  {
	      $themes{$name} = $theme;
	  }
      }
  }
  closedir(DIR);
  return %themes;
}

sub cw_duplicate_theme
{
    my $id = shift;
    my $newid = shift;
    my $theme;
    my $wcald = cw_get_wcald();
    if ($id)
    {
	system("cp -r $wcald->{DIRECTORY}/themes/$id $wcald->{DIRECTORY}/themes/$newid");
    }
    else
    {
	system("cp -r $wcald->{DIRECTORY}/images $wcald->{DIRECTORY}/themes/$newid");
    }
}

sub cw_delete_theme
{
    my $id = shift;
    my $theme;
    my $wcald = cw_get_wcald();
    system("\\rm -rf $wcald->{DIRECTORY}/themes/$id");
}

sub cw_rename_theme
{
    my $id = shift;
    my $newid = shift;
    my $theme;
    my $wcald = cw_get_wcald();
    rename("$wcald->{DIRECTORY}/themes/$id", "$wcald->{DIRECTORY}/themes/$newid");
}

sub cw_export_theme
{
    my $id = shift;
    my $theme;
    my $wcald = cw_get_wcald();
    system("cd $wcald->{DIRECTORY}/themes; tar cvf $id.tar $id > /dev/null; gzip $id.tar");
    print 'Content-Type: application/binary; name="'.$id.'.tgz"'."\n";
    print 'Content-Disposition: inline; filename="'.$id.'.tgz"'."\n\n";
    open(FILE, "$wcald->{DIRECTORY}/themes/$id.tar.gz");
    print <FILE>;
    close(FILE);
    unlink("$wcald->{DIRECTORY}/themes/$id.tar.gz");
}

sub cw_bgstr
{
    my $bgt = shift;
    my $theme = shift;
    my $www = shift;
    my $str = "";
    my $wconf = cw_get_wconf();
    my $url = $wconf->{"base url"} || "/wcal";
    if ($www)
    {
	$url = ".";
    }
    elsif ($theme)
    {
	$url .= "/themes/$theme->{id}";
    }
    else
    {
	$url .= "/images";
    }

    if ($bgt == 1)
    {
	if ($theme)
	{
	    if ($theme->{tdcolor})
	    {
		$str = " bgcolor=$theme->{tdcolor} ";
	    }
	    else
	    {
		$str = " bgcolor=#D9F4F4 ";
	    }
	    if ($theme->{tdbg})
	    {
		$str .= "background=\"$url/$theme->{tdbg}\" ";
	    }
	}
	else
	{
	    $str = " bgcolor=#D9F4F4 ";
	}
    }
    elsif ($bgt == 2)
    {
	if ($theme)
	{
	    if ($theme->{thcolor})
	    {
		$str = " bgcolor=$theme->{thcolor} ";
	    }
	    else
	    {
		$str = " bgcolor=#E2E3FC ";
	    }
	    if ($theme->{thbg})
	    {
		$str .= "background=\"$url/$theme->{thbg}\" ";
	    }
	}
	else
	{
	    $str = " bgcolor=#E2E3FC ";
	}
    }
    elsif ($bgt == 3)
    {
	if ($theme)
	{
	    if ($theme->{td1color})
	    {
		$str = " bgcolor=$theme->{td1color} ";
	    }
	    else
	    {
		$str = " bgcolor=#F2F3FC ";
	    }
	    if ($theme->{td1bg})
	    {
		$str .= "background=\"$url/$theme->{td1bg}\" ";
	    }
	}
	else
	{
	    $str = " bgcolor=#F2F3FC ";
	}
    }
    elsif ($bgt == 4)
    {
	if ($theme)
	{
	    if ($theme->{td2color})
	    {
		$str = " bgcolor=$theme->{td2color} ";
	    }
	    else
	    {
		$str = " bgcolor=#FCF3F2 ";
	    }
	    if ($theme->{td2bg})
	    {
		$str .= "background=\"$url/$theme->{td2bg}\" ";
	    }
	}
	else
	{
	    $str = " bgcolor=#FCF3F2 ";
	}
    }
    elsif ($bgt == 5)
    {
	if ($theme)
	{
	    if ($theme->{nowtdcolor})
	    {
		$str = " bgcolor=$theme->{nowtdcolor} ";
	    }
	    else
	    {
		$str = " bgcolor=#CAFFD0 ";
	    }
	    if ($theme->{nowtdbg})
	    {
		$str .= "background=\"$url/$theme->{nowtdbg}\" ";
	    }
	}
	else
	{
	    $str = " bgcolor=#CAFFD0 ";
	}
    }
    elsif ($bgt == 6)
    {
	if ($theme)
	{
	    if ($theme->{sdayscolor})
	    {
		$str = " bgcolor=$theme->{sdayscolor} ";
	    }
	    else
	    {
		$str = " bgcolor=#E0E0E0 ";
	    }
	    if ($theme->{sdaysbg})
	    {
		$str .= "background=\"$url/$theme->{sdaysbg}\" ";
	    }
	}
	else
	{
	    $str = " bgcolor=#E0E0E0 ";
	}
    }
    elsif ($bgt == 7)
    {
	if ($theme)
	{
	    if ($theme->{nowthcolor})
	    {
		$str = " bgcolor=$theme->{nowthcolor} ";
	    }
	    else
	    {
		$str = " bgcolor=#CAFFD0 ";
	    }
	    if ($theme->{nowthbg})
	    {
		$str .= "background=\"$url/$theme->{nowthbg}\" ";
	    }
	}
	else
	{
	    $str = " bgcolor=#CAFFD0 ";
	}
    }
    elsif ($bgt == 8)
    {
	if ($theme)
	{
	    if ($theme->{backcolor})
	    {
		$str = " bgcolor=$theme->{backcolor} ";
	    }
	    else
	    {
		$str = " bgcolor=#FFFFFF ";
	    }
	    if ($theme->{backbg})
	    {
		$str .= "background=\"$url/$theme->{backbg}\" ";
	    }
	}
	else
	{
	    $str = " bgcolor=#FFFFFF background=\"$url/background.jpg\"";
	}
    }
    elsif ($bgt == 9)
    {
	if ($theme)
	{
	    if ($theme->{mwdcolor})
	    {
		$str = " bgcolor=$theme->{mwdcolor} ";
	    }
	    else
	    {
		$str = " bgcolor=#FFFFFF ";
	    }
	    if ($theme->{mwdbg})
	    {
		$str .= "background=\"$url/$theme->{mwdbg}\" ";
	    }
	}
	else
	{
	    $str = " bgcolor=#FFFFFF background=\"$url/mwdbackground.jpg\"";
	}
    }

    return $str;
}

# MAIN *************************************************

# Read cwcal.conf
my $line;
my $key;
my $val;

open(CONF, "inc/cwcal.conf") or die "Can't open cwcal.conf for reading: $!";
while ($line = <CONF>)
{
    if ($line =~ /\s*([^=]*\S)\s*=\s*(.*\S)/)
    {
	$key = $1;
	$val = $2;
	if ($key eq "lang")
	{
	    $::REMOTE_LANG = $val;
	}
	elsif ($key eq "wcal bin")
	{
	    $::WCAL_BIN = $val;
	}
	else
	{
	    die "Unknown '$key' resource";
	}
    }
}
close(CONF);

# Read cwcal.msg
open(MSG, "inc/cwcal.msg") or die "Can't open cwcal.msg for reading: $!";
my $old_slash = $/;
undef $/;
$::msgs = eval (<MSG>);
$/ = $old_slash;
close(MSG);
